==> facility-2.25.6-linux-amd64.tar.zst: release-v2.25.6/packages/facility-server/app/database/integrity.js <==
import config from 'config';
import { log } from '@tamanu/shared/services/logging';
import { isSyncTriggerDisabled } from '@tamanu/database/dataMigrations';
import { selectFacilityIds } from '@tamanu/utils/selectFacilityIds';
import { CentralServerConnection } from '../sync';

export async function performDatabaseIntegrityChecks(context) {
  if (await isSyncTriggerDisabled(context.sequelize)) {
    throw Error('Sync Trigger is disabled in the database.');
  }

  // run in a transaction so any errors roll back all changes
  await context.sequelize.transaction(async () => {
    await ensureHostMatches(context);
    await ensureFacilityMatches(context);
  });
}

/*
 * ensureHostMatches
 */
async function ensureHostMatches(context) {
  const { LocalSystemFact } = context.models;
  const centralServer = new CentralServerConnection(context);
  const configuredHost = centralServer.host;
  const lastHost = await LocalSystemFact.get('syncHost');

  if (!lastHost) {
    await LocalSystemFact.set('syncHost', centralServer.host);
    return;
  }

  if (lastHost !== configuredHost) {
    throw new Error(
      `integrity check failed: sync.host mismatch: read ${configuredHost} from config, but already connected to ${lastHost} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "syncHost" key from the "local_system_fact" table)`,
    );
  }
}

/*
 * ensureFacilityMatches
 */
async function ensureFacilityMatches(context) {
  const { LocalSystemFact } = context.models;
  const configuredFacilities = selectFacilityIds(config);
  const lastFacilities = await LocalSystemFact.get('facilityIds');

  if (!lastFacilities) {
    await performInitialIntegritySetup(context);
    return;
  }

  // ensure both arrays contain the same set of facility ids
  const match = JSON.parse(lastFacilities).every((facilityId) =>
    configuredFacilities.includes(facilityId),
  );
  if (!match) {
    // if the facility doesn't match, error
    throw new Error(
      `integrity check failed: serverFacilityId mismatch: read ${configuredFacilities} from config, but already registered as ${lastFacilities} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "facilityIds" key from the "local_system_fact" table)`,
    );
  }
}

async function performInitialIntegritySetup(context) {
  const centralServer = new CentralServerConnection(context);
  log.info(`Verifying sync connection to ${centralServer.host}...`);

  const { token, serverFacilityIds } = await centralServer.connect();

  if (!token) {
    throw new Error('Could not obtain valid token from central server.');
  }

  // We've ensured that our immutable config stuff is valid -- save it!
  const { LocalSystemFact } = context.models;
  const facilityIdsString = JSON.stringify(serverFacilityIds);
  await LocalSystemFact.set('facilityIds', facilityIdsString);

  log.info(`Verified with central server as facilities ${facilityIdsString}`);
}

==> facility-2.35.5-linux-amd64.tar.zst: release-v2.35.5/packages/facility-server/app/database/integrity.js <==
import config from 'config';
import { FACT_CENTRAL_HOST, FACT_FACILITY_IDS } from '@tamanu/constants/facts';
import { log } from '@tamanu/shared/services/logging';
import { isSyncTriggerDisabled } from '@tamanu/database/dataMigrations';
import { selectFacilityIds } from '@tamanu/utils/selectFacilityIds';
import { CentralServerConnection } from '../sync';

export async function performDatabaseIntegrityChecks(context) {
  if (await isSyncTriggerDisabled(context.sequelize)) {
    throw Error('Sync Trigger is disabled in the database.');
  }

  // run in a transaction so any errors roll back all changes
  await context.sequelize.transaction(async () => {
    await ensureHostMatches(context);
    await ensureFacilityMatches(context);
  });
}

/*
 * ensureHostMatches
 */
async function ensureHostMatches(context) {
  const { LocalSystemFact } = context.models;
  const centralServer = new CentralServerConnection(context);
  const configuredHost = centralServer.host;
  const lastHost = await LocalSystemFact.get(FACT_CENTRAL_HOST);

  if (!lastHost) {
    await LocalSystemFact.set(FACT_CENTRAL_HOST, centralServer.host);
    return;
  }

  if (lastHost !== configuredHost) {
    throw new Error(
      `integrity check failed: sync.host mismatch: read ${configuredHost} from config, but already connected to ${lastHost} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "syncHost" key from the "local_system_fact" table)`,
    );
  }
}

/*
 * ensureFacilityMatches
 */
async function ensureFacilityMatches(context) {
  const { LocalSystemFact } = context.models;
  const configuredFacilities = selectFacilityIds(config);
  const lastFacilities = await LocalSystemFact.get(FACT_FACILITY_IDS);

  if (!lastFacilities) {
    await performInitialIntegritySetup(context);
    return;
  }

  // ensure both arrays contain the same set of facility ids
  const match = JSON.parse(lastFacilities).every((facilityId) =>
    configuredFacilities.includes(facilityId),
  );
  if (!match) {
    // if the facility doesn't match, error
    throw new Error(
      `integrity check failed: serverFacilityId mismatch: read ${configuredFacilities} from config, but already registered as ${lastFacilities} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "facilityIds" key from the "local_system_fact" table)`,
    );
  }
}

async function performInitialIntegritySetup(context) {
  const centralServer = new CentralServerConnection(context);
  log.info(`Verifying sync connection to ${centralServer.host}...`);

  const { token, serverFacilityIds } = await centralServer.connect();

  if (!token) {
    throw new Error('Could not obtain valid token from central server.');
  }

  // We've ensured that our immutable config stuff is valid -- save it!
  const { LocalSystemFact } = context.models;
  const facilityIdsString = JSON.stringify(serverFacilityIds);
  await LocalSystemFact.set(FACT_FACILITY_IDS, facilityIdsString);

  log.info(`Verified with central server as facilities ${facilityIdsString}`);
}
